<?php

// QuickTalk guestbook 4.0 build:20200514

// Note:

// Here after
// $host is the path to the repository directory
// $root is the repository directory (i.e. $host + "data/")
// $path is the root + the year and month sub-directories
// When using the default installation directory, $host remains "" (empty string) and $root is thus simply "data/".

// If you want to store the messages in an other directory:
// the fullpath of your storage location must be declared in the qtg_init.php file: in the variable $qtg_host (with final /)
// Example: To store data in //storage/qtg/data/ you must declare $qtg_host = "//storage/qtg/" in the qtg_init.php file

class cDT
{

public $id = -1;
public $type = 'file';
public $host;
public $root;
public $path;
public $arrFiles = array(); // list of files (without .txt)
public $curFiles = 0; // cursor to list of files
public $lastfile = ''; // most recent
public $firstfile = ''; // oldest

public function __construct($qtg_host='')
{
  $this->host = $qtg_host; // path to the repository (with final /). This value comes from the qtg_init.php file
  $this->root = $this->host.'data/';
  $this->path = $this->root.date('Y').'/'.date('m').'/';
}

public function FilesCount($option='',$intMax=0,$strStart='0',$strEnd='0',$stats=false,$append=false,$order=false)
{
  if ( !is_string($option) ) die('FilesCount:#1 must be a string');
  // Creates list of filenames
  $this->GetFiles($option,$intMax,(string)$strStart,(string)$strEnd,$append);
  // Optional reordering (GetFiles already returns filenames in desc order)
  if ( $order==='desc' ) rsort($this->arrFiles);
  if ( $order==='asc' ) sort($this->arrFiles);
  // Optional stats on filenames
  if ( count($this->arrFiles)>0 && $stats )
  {
  $this->lastfile = reset($this->arrFiles); // most recent is the first filename in the list
  $this->firstfile = end($this->arrFiles); //oldest is the last filename in the list
  }
  // Count
  return count($this->arrFiles);
}

public function Getrows($max=999)
{
  $rows=array();
  $i=0;
  while($row=$this->Getrow()) { $rows[]=$row; ++$i; if ( $i===$max ) break; }
  return $rows;
}

public function Getrow()
{
  if ( !isset($this->arrFiles[$this->curFiles]) ) return false;

  $row = array();
  $strId = $this->arrFiles[$this->curFiles];

  $arrFile = explode('-',$strId);
  $strPath = $this->GetPath($strId);
  $row['id'] = $this->curFiles;
  $row['ip'] = str_replace('w',':',$arrFile[2]); // ipv6 block separator MUST be ':' (while filename use 'w')
  $row['issuedate'] = $arrFile[0].$arrFile[1];
  $row['visible'] = strtoupper($arrFile[3]);
  // Read file
  $str = file_get_contents($strPath.$strId.'.txt');
  $arrLines = explode("\n",$str,8);
  foreach($arrLines as $strLine)
  {
    $strLine = trim($strLine); // required as the line can start with \n
    if ( substr($strLine,0,4)==='name' )      { $row['name'] = trim(substr($strLine,4)); continue; }
    if ( substr($strLine,0,7)==='userage' )   { $row['userage'] = trim(substr($strLine,7)); continue; }
    if ( substr($strLine,0,9)==='useremail' ) { $row['useremail'] = trim(substr($strLine,9)); continue; }
    if ( substr($strLine,0,8)==='postfrom' )  { $row['postfrom'] = trim(substr($strLine,8)); continue; }
    if ( substr($strLine,0,1)==='x' )         { $row['x'] = trim(substr($strLine,1)); continue; }
    if ( substr($strLine,0,1)==='y' )         { $row['y'] = trim(substr($strLine,1)); continue; }
    if ( substr($strLine,0,1)==='z' )         { $row['z'] = trim(substr($strLine,1)); continue; }
    if ( substr($strLine,0,7)==='message' )   $row['message'] = trim(substr($strLine,7));
  }
  ++$this->curFiles;
  return $row;
}

public function GetPath($id)
{
  if ( !is_string($id) ) die('GetPath: arg #1 must be a string');
  return $this->root.substr($id,0,4).'/'.substr($id,4,2).'/';
}

public function GetFiles($option='',$intMax=0,$strStart='0',$strEnd='0',$append=false,$order='desc')
{
  // This create a list of filenames between start(recent) and end(old) dates (maximum 1000 messages)
  // With $append=true, it can also append filenames to existing list [arrFiles]
  // $option can be '' (all messages), 'y', 'n', 'u' (visible status) or a csv of 2 status. Ex: 'n;u'
  // Note: with file system 'n;u' becomes to '-n' or '-u'

  $options = array_filter(array_map('trim',explode(';',$option)));

  // Reset arrFiles if not append
  if ( !$append ) $this->arrFiles = array();

  // Read from most recent to older in the directories. Return a list of file id (without .txt)
  if ( empty($strStart) ) $strStart=date('Ym');
  if ( strlen($strStart)===4 ) $strStart.='12';
  $strStart = substr($strStart,0,6); // must be 6 char
  if ( empty($strEnd) ) $strEnd='201001';
  if ( strlen($strEnd)===4 ) $strEnd.='01';
  $strEnd = substr($strEnd,0,6); // must be 6 char
  $intYstart = (int)substr($strStart,0,4);
  $intYend = (int)substr($strEnd,0,4);
  $intMstart = (int)substr($strStart,4,2);
  $intMend = (int)substr($strEnd,4,2);
  $i=0;

  for($intY=$intYstart;$intY>=$intYend;$intY--)
  {
    $strY = (string)$intY;
    if ( !is_dir($this->root.$strY) ) continue;
    for($intM=12;$intM>0;$intM--)
    {
      $strM = substr('00'.$intM,-2);
      if ( $strY.$strM>$strStart ) continue;
      if ( !is_dir($this->root.$strY.'/'.$strM) ) continue;
      $strPath = $this->root.$strY.'/'.$strM.'/';
      $handle = opendir($strPath);
      while(false!==($file=readdir($handle)))
      {
        if ( $file=='.' || $file=='..' ) continue;
        if ( substr($file,-4,4)!='.txt' ) continue;
        if ( $intMax>0 && $i>$intMax ) { closedir($handle); return $this->arrFiles;} // this breaks
        if ( $i>1000 )  { closedir($handle); return $this->arrFiles; } // this breaks
        $file = substr($file,0,-4);
        if ( empty($option) )  { $this->arrFiles[]=$file; continue; }
        // check options
        foreach($options as $str)
        {
          if ( empty($str) ) continue;
          $str = strtolower($str);
          if ( $str==='u' || $str==='n' || $str==='y' ) $str = '-'.$str;
          if ( strstr($file,$str) ) { $this->arrFiles[]=$file; break; }
        }
        $i++;
        if ( $strY.$strM===$strEnd ) break;
      }
      closedir($handle);
    }
  }
  // reodering desc [default] is mandatory to get day/hours sorted
  if ( $order==='desc' ) rsort($this->arrFiles);
  if ( $order==='asc' ) sort($this->arrFiles);
  return $this->arrFiles;
}

public function GetFilesSubset($option='',$intMax=0,$strStart='0',$strEnd='0',$order=false)
{
  // This create a subset of $this->arrFiles for matching option(s) and dates between start(recent) and end(old)
  // $option can be '' (all messages), 'y', 'n', 'u' (visible status) or a csv of 2 status. Ex: 'n;u'
  // Note: with file system 'n;u' becomes to '-n' or '-u'
  if ( empty($this->arrFiles) ) return array();
  $options = array_filter(array_map('trim',explode(';',$option)));
  if ( is_int($strStart) ) $strStart = (string)$strStart;
  if ( is_int($strEnd) ) $strEnd = (string)$strEnd;

  // Read from most recent to older in the directories. Return a list of file id (without .txt)
  if ( empty($strStart) ) $strStart=date('Ymd');
  if ( strlen($strStart)===4 ) $strStart.='1231';
  if ( strlen($strStart)===6 ) $strStart.='31';
  $intStart = (int)substr($strStart,0,8); // YYYYMMDD (new)
  if ( empty($strEnd) ) $strEnd='20000101';
  if ( strlen($strEnd)===4 ) $strEnd.='0101';
  if ( strlen($strEnd)===6 ) $strEnd.='01';
  $intEnd = (int)substr($strEnd,0,8); //YYYYMMDD (old)
  $i=0;
  $arr = array();
  foreach($this->arrFiles as $file)
  {
    if ( $intMax>0 && $i>$intMax ) break;
    $fileYMD = (int)substr($file,0,8); if ( $fileYMD<$intEnd || $fileYMD>$intStart ) continue;
    if ( empty($option) ) { $arr[]=$file; continue; }
    foreach($options as $str)
    {
      if ( empty($str) ) continue;
      $str = strtolower($str);
      if ( $str==='u' || $str==='n' || $str==='y' ) $str = '-'.$str;
      if ( strstr($file,$str) ) { $arr[]=$file; break; }
    }
    ++$i;
  }
  // reodering (default $order=false because the source arrFiles is already sorted)
  if ( $order==='desc' ) rsort($arr);
  if ( $order==='asc' ) sort($arr);
  return $arr;
}

public static function MakeDir($strPath,$b12=false)
{
  // Use $b12=true to check/create 12 subdirectories for the year
  // Returns '' (empty string), or a message in case of error
  if ( !is_string($strPath) ) die('MakeDir: arg #1 must be a string');
  if ( substr($strPath,-1,1)==='/' ) $strPath =  substr($strPath,0,-1);

  // Check dir
  if ( is_dir($strPath) ) return "Cannot create already existing directory: $strPath";

  // Search the root "data"-directory (i.e. find the last "data" in path)
  $arr = explode('/',$strPath);
  $arrData = array_keys($arr, 'data'); // find all "data" in the path
  if ( empty($arrData) ) return "Cannot find the data directory in $strPath";
  $intData = end($arrData); // find last "data"

  // Get the year and month dir in the path
  if ( !isset($arr[$intData+1]) ) return "Cannot find the year directory in $strPath";
  if ( !isset($arr[$intData+2]) ) return "Cannot find the month directory in $strPath";
  $strYear = $arr[$intData+1]; // year dir (next to the last data dir)
  $strMonth = $arr[$intData+2]; // month dir (next to the year dir)
  $strRoot = implode('/',array_slice($arr, 0, $intData+1)); // group all parts before the year

  // Check the year directory
  if ( !is_dir($strRoot.'/'.$strYear) )
  {
     if ( !mkdir($strRoot.'/'.$strYear) ) return "Cannot create the directory: $strRoot/$strYear"; // make the year directory
  }

  // Check the month(s) directory
  $arrMonths = $b12 ? array('01','02','03','04','05','06','07','08','09','10','11','12') : array($strMonth);
  foreach ($arrMonths as $strMonth)
  {
    if ( !is_dir($strRoot.'/'.$strYear.'/'.$strMonth) )
    {
     if ( !mkdir($strRoot.'/'.$strYear.'/'.$strMonth) ) return "Cannot create the directory: $strRoot/$strYear/$strMonth"; // make the month directory
    }
  }
  return '';
}


public function getSetting(string $key='', string $dflt='')
{
  // Returns the db setting [string] for param $key, or $dflt if $key does not existing
  if ( empty($key) ) die('CDT::getSetting invalid key');
  // CDT uses session instead of DB
  return isset($_SESSION[QT][$key]) ? (string)$_SESSION[QT][$key] : $dflt;
}

}