<?php

/* ============
 * version: 4.0 build:20200514
 * This is a library of public class
 * ------------
 * class cStatOptions
 * QTroof()
 * QTcumul()
 * QTpercent()
 * QTarrayzero()
 * ------------
 * NOTE:
 * the function row_count have been removed because it is not supported by odbc
 * ============ */
 
class cStatOptions
{
  public $type='b'; // bar, line (b,l)
  public $gpc=false; // graph values in percent
  public $dpc=false; // diff values in percent
  public $plot='y'; // y to plot 2years, v to plot variation values, a to plot both
  public function setFromStr($str)
  {
    if ( !is_string($str) ) die('cStatOptions:setFromStr argument must be a string');
    if ( isset($str[0]) ) $this->type = $str[0]==='l' ? 'l' : 'b';
    if ( isset($str[1]) ) $this->gpc = (bool)$str[1];
    if ( isset($str[2]) ) $this->dpc = (bool)$str[2]; 
    if ( isset($str[3]) ) $this->plot = strtolower($str[3]);
  }
  public function asStr()
  {
    return ($this->type=='l' ? 'l' : 'b').(int)$this->gpc.(int)$this->dpc.$this->plot;
  }
}

function QTarrayzero($arr,$dropkeys=true)
{
  // Change empty (or null) values to 0 in the array
  if ( !is_array($arr) ) die('QTarrayzero: Arg #1 must be an array');
  foreach($arr as $key=>$value) if ( empty($value) ) $arr[$key]=0;
  return $dropkeys ? array_values($arr) : $arr;
}

function QTroof($arr)
{
  // Returns 5,10,20,30,50,100,200,500,1000 or n000 ABOVE the maximum value in the serie.
  // $arr can a a single number
  if ( is_numeric($arr) ) $arr = array($arr);
  if ( !is_array($arr) ) die('QTroof: Arg #1 must be an array');
  $intTop = 5;
  $i = max($arr);
  if ( $i>1000 ) return (floor($i/1000)+1)*1000;
  if ( $i>500 ) return 1000;
  if ( $i>5 ) $intTop = 10;
  if ( $i>10 ) $intTop = 20;
  if ( $i>20 ) $intTop = 30;
  if ( $i>30 ) $intTop = 50;
  if ( $i>50 ) $intTop = 100;
  if ( $i>100 ) $intTop = 200;
  if ( $i>200 ) $intTop = 500;
  return $intTop;
}

function QTcumul($arr,$d=0)
{
  // Cumulate the values in the array: 1,2,3,2,1 becomes 1,3,6,8,9
  // $d defines the number of decimals
  // Note: Result is an array of float values
  if ( !is_array($arr) ) die ('QTcumul: Arg #1 must be an array');
  $arrC = array();
  $i=0;
  foreach($arr as $strKey=>$aValue)
  {
    $i += $aValue;
    $arrC[$strKey]=round($i,$d);
  }
  return $arrC;
}

function QTpercent($arr,$d=0,$b=true)
{
  // Returns the percentage of each value in the serie: 100,20,30,50 becomes 50,10,15,25 (%)
  // $d defines the number of decimals
  // $b set TRUE to get a percent value (e.g.: 50 in unit %). Use FALSE to get de ratio (e.g.: 0.5)
  // if $arr contains 0 (or any empty value) that percentage will be 0
  // Note: Result is an array of float values
  if ( !is_array($arr) ) die ('QTpercent: Arg #1 must be an array');
  if ( !is_int($d) ) die ('QTpercent: Arg #2 must be an integer');
  if ( !$b && $d<1 ) $d=1; // ratio must have at least 1 decimal
  $arrP = array();
  $intTotal = array_sum($arr);
  foreach($arr as $strKey=>$oValue)
  {
    $i = (empty($oValue) ? 0 : $oValue/$intTotal);
    if ( $b ) $i = $i*100;
    $arrP[$strKey]=round($i,$d);
  }
  return $arrP;
}

function SerieExtract($key,$a,$b=array(null))
{
  // Returns the $key array inside the $a serie (values are unindexed)
  // If $b is an array containing also a $key-value, this value is added to the array (push)
  // Note: returns an empty array if the $key is not found
  if ( !is_array($a) || !is_array($b) ) die('GetArrDiff: invalid arguments #2 or #3');
  $arr = array();
  if ( isset($a[$key]) )
  {
    if ( !is_array($a[$key]) ) $a[$key] = array($a[$key]);
    $arr = array_values($a[$key]);
    if ( isset($b[$key]) ) array_push($arr,$b[$key]);
  }
  return $arr;
}

function GetArrDiff($a,$b,$percent=0,$round=-1,$null=0)
{
  // Compute difference (or percentage diff) between 2 arrays in sequential order (array keys are not used) 
  if ( is_numeric($a) ) $a = array($a);
  if ( is_numeric($b) ) $b = array($b);
  if ( $percent===false ) $percent=0;
  // $percent 0 to skip percentage, use 1 or 100 to get ratio or % (0.5 or 50)
  // $round -1 to skip round
  // return $null if value is not set in $a or $b
  if ( !is_array($a) || !is_array($b) ) die('GetArrDiff: invalid arguments #1 or #2');
  if ( !is_numeric($percent) || $percent<0 ) die('GetArrDiff: invalid#3');
  if ( !is_int($round) ) die('GetArrDiff: invalid#4');
  $arr = array();
  $a = array_values($a); // drops array keys
  $b = array_values($b);
  for($i=0;$i<count($a);++$i)
  {
    $arr[$i] = is_numeric($a[$i]) && is_numeric($b[$i]) ? $b[$i]-$a[$i] : $null;
    if ( !is_numeric($arr[$i]) ) continue;
    if ( $percent>0 && $arr[$i]!==0 ) $arr[$i] = $a[$i]===0 ? $arr[$i]*$percent : $arr[$i]*$percent/$a[$i];
    if ( $round>=0 ) $arr[$i] = round($arr[$i],$round);
  }
  return $arr;
}

function GetAbscise($strTime='m',$intMaxBt=13,$strTendaysago=-10)
{
  $arr = array();
  switch($strTime)
  {
  case 'q': for ($i=1;$i<=$intMaxBt;++$i) { $arr[$i]='Q'.$i; } break;
  case 'm': for ($i=1;$i<=$intMaxBt;++$i) { $arr[$i]=substr(L('dateMM.'.$i),0,2); } break; // 2 chars only
  case 'd': for ($i=1;$i<=$intMaxBt;++$i) { $arr[$i]=substr(DateAdd($strTendaysago,$i,'day'),-2,2); } break;
  }
  return $arr;
}