<?php
/**
* PHP versions 4 and 5
*
* LICENSE: This source file is subject to version 3.0 of the PHP license
* that is available through the world-wide-web at the following URI:
* http://www.php.net/license.  If you did not receive a copy of
* the PHP License and are unable to obtain it through the web, please
* send a note to license@php.net so we can mail you a copy immediately.
*
* @package    QuickTalk Guestbook
* @author     Philippe Vandenberghe <info@qt-cute.org>
* @copyright  2009-2012 The PHP Group
* @version    4.0.3 build:20220105
*/

if (session_status()==PHP_SESSION_NONE) session_start();
require 'qtg_init.php';

$oP->selfurl='qtg_index.php';
if ( $qtg_gkey )
{
include 'qtg_map_lib.php';
include 'language/'.$_SESSION[QT]['language'].'/qtg_map.php';
}

$bCol = $qtg_column_default;
$limit = 0;
$page = 1;

// GET parametres
if ( isset($_GET['page']) ) { $page = (int)$_GET['page']; $limit = ($page-1)*$qtg_page_size; }
if ( $page<1 ) $page = 1;

// Check fallback cache format
if ( !MEMCACHE_HOST && MEMCACHE_FALLBACK )
{
if ( file_exists(MEMCACHE_FALLBACK.QT.'-countY'.(QT_FILE ? 'row' : 'file').'.json') ) {  memcacheFlush(); $oHtml->showlog[]='Warning: Memcache purged due to format change.'; }
}

// COUNT MESSAGES (visible)

$intMessages = 0;
if ( QT_FILE )
{
  $arr = memcacheGet(QT.'-countYfile'); // must be [array] filenames
  if ( is_array($arr) )
  {
    $oDB->arrFiles = $arr;
    $intMessages = count($oDB->arrFiles);
  }
  else
  {
    memcacheFlush(); // filnames never created (or system changed): need to clear cache
    $intMessages=CountMessages($oDB,'y'); // this creates filenames as $oDB->arrFiles
    memcacheSet(QT.'-countYfile',$oDB->arrFiles); // store the filenames
  }
}
else
{
  $intMessages = memcacheGet(QT.'-countYrow');
  if ( $intMessages===false )
  {
    memcacheFlush(); // filnames never created (or system changed): need to clear cache
    $intMessages = CountMessages($oDB,'y');
    memcacheSet(QT.'-countYrow',$intMessages);
  }
}

// --------
// HTML START
// --------

include 'qtg_inc_hd.php';

// TIPS WELCOME and DISPLAY COMMAND

$str = L('Welcome');
if ( file_exists('config/welcome-'.$_SESSION[QT]['language'].'.txt') ) $str = file_get_contents('config/welcome-'.$_SESSION[QT]['language'].'.txt');
$str = trim($str); // required to remove eof char
echo '
<div class="ctrl',(empty($str) ? ' voidwelcome' : ''),'">';
if ( $qtg_home<>'' ) echo '<a class="home" href="',$qtg_home,'" title="',Lh('Home'),'"><i class="qtg-icon icon-home"></i></a>';
if ( $qtg_column_change ) echo '<a href="javascript:void(0)" title="',Lh('View_std'),'" onclick="changeColumn(1);return false;"><i class="qtg-icon icon-menu"></i></a><a href="javascript:void(0)" title="',Lh('View_col'),'" onclick="changeColumn(2);return false;"><i class="qtg-icon icon-th-large"></i></a>';
echo '</div>
<div class="welcome',(empty($str) ? ' voidwelcome' : ''),'">',$str,'</div>

<div class="content">
';

if ( $qtg_actif && $qtg_separateform ) echo '<div class="button sign"><a href="',Href('qtg_msg_add.php').'">',L('Sign'.$qtg_type),'</a></div>';

// -------------
// SHOW MESSAGES AND MAP
// -------------

// check if memcache $out and $arrCoord are usable (page 1)

$useMem = false;
$out = '';
$map = '';
$arrCoord = array();
if ( $page===1 && $out=memcacheGet(QT.'-page1'.$_SESSION[QT]['language'],false) ) $useMem=true;
if ( $useMem && $qtg_gkey )
{
  if ( !$arrCoord=memcacheGet(QT.'-coord') ) { $arrCoord = array(); $useMem=false; }
}

// use memcache (page1) or build the page

if ( $useMem )
{
  echo $out;
}
else
{
  if ( $intMessages===0 )
  {
    $out = '<p class="disabled">No message</p>';
  }
  else
  {
    // Pager
    $pager = MakePager(Href($oP->selfurl),$intMessages,$qtg_page_size,$page);
    $pagertop = empty($pager) ? '' : '<div class="pager top">'.L('Page').$pager.'</div>'.PHP_EOL;
    $pagerbot = empty($pager) ? '' : '<div class="pager bot">'.L('Page').$pager.'</div>'.PHP_EOL;
    if ( QT_FILE )
    {
      // $oDB->arrFiles is already created (during count step)
      if ( count($oDB->arrFiles)>$qtg_page_size ) $oDB->arrFiles = array_slice($oDB->arrFiles, $limit, $qtg_page_size);
      $oDB->curFiles=0;
    }
    else
    {
      $sql = LimitSQL('* FROM '.TABMESSAGE.' WHERE id>=0 AND visible="Y"','issuedate DESC',$limit,$qtg_page_size);
      $oDB->Query($sql);
    }
    $rows = $oDB->Getrows($qtg_page_size);
    // Messages column(s)
    $out = '<div id="messages" class="messages column'.($bCol ? 's' : '').'">
    ';
    $iCol = 1;
    $iMsg = 0;
    $newCoord = false;
    while($iMsg<$qtg_page_size)
    {
      if ( !isset($rows[$iMsg]) || !is_array($rows[$iMsg]) ) break;
      $oPost = new cPost($rows[$iMsg]);
      $hid = cPost::Hash($oPost->id); //ouput must show hashed id
      $strEdition = '';
      // register map point
      if ( $qtg_gkey && !isset($arrCoord[$hid]) ) {
      if ( $oPost->x!=0 && $oPost->y!=0 ) {
        $strPname = QTattr($oPost->name,40);
        $strPlink = '<a href="http://maps.google.com?q='.$oPost->y.','.$oPost->x.'+('.$strPname.')" class="small" title="'.$L['map_In_google'].'" target="_blank">[G]</a>';
        $strPtxt = QTattr(QTinline($oPost->message,150,' ')).($qtg_use_from && !empty($oPost->postfrom) ? '<br>'.QTattr($oPost->postfrom) : '');
        $strPtxt = str_replace("\r\n"," ",$strPtxt);
        $strPtxt = str_replace("\n"," ",$strPtxt);
        if ( $qtg_bbc_smiley ) $strPtxt = RenderSmile($strPtxt,$qtg_tags,$qtg_skin);
        $aPoint = MakePoint( $oPost->y, $oPost->x, $oPost->name, '<div class="ginfo"><p>'.QTattr($strPname).'</p><p>'.$strPtxt.'</p><p>'.QTdatestr($oPost->issuedate,FORMATDATE,FORMATTIME,true).' '.$strPlink.'</p></div>', (isset($qtg_gsymbol) ? $qtg_gsymbol : false) );
        $arrCoord[$hid] = $aPoint;
        if ( $page===1 ) $newCoord=true;
      }}
      // ------------
      $intMsg = strlen($oPost->message);
      $out .= '<div class="msg">'.PHP_EOL;
      $out .= '<p class="msg-header">'.RenderMsgHeader($oPost,$qtg_use_from).'</p>'.PHP_EOL;
      $out .= '<p class="msg-body" id="msg-'.$hid.'">'.RenderMsg($oPost->message,$qtg_show_size,$qtg_tags,'<small>...</small>',$qtg_skin).MoreLink($intMsg,$qtg_show_size,$hid,$page).'</p>'.PHP_EOL;
      $out .= '<p class="msg-footer">'.RenderMsgFooter($oPost,$qtg_use_email,$qtg_use_age,$qtg_gkey,true,true).'</p>'.PHP_EOL;
      $out .= '</div>'.PHP_EOL;
      if ( $intMsg>$qtg_show_size && $intMsg<$qtg_show_size*3 && $intMsg<801 ) $out .= '<div style="display:none" id="fullmsg-'.$hid.'">'.RenderMsg($oPost->message,800,$qtg_tags,'',$qtg_skin).' <a class="more-link" href="javascript:void(0);" onclick="msgSwitch(\''.$hid.'\');">'.L('Less').'</a></div>'.PHP_EOL;
      // ------------
      ++$iMsg;
    }
    $out .= '</div>'.PHP_EOL;

    if ( $qtg_gkey )
    {
      // memcache arrCoord
      if ( $newCoord ) memcacheSet(QT.'-coord',$arrCoord);

      // Map
      if ( count($arrCoord)==0 )
      {
        $map = '<p class="mapempty">'.L('map_E_noposition').'</p>'.PHP_EOL;
      }
      else
      {
        //select zoomto (maximum 20 items in the list)
        $str = '';
        if ( count($arrCoord)>1 )
        {
          $str = '<p class="gmap top"><a href="javascript:void(0)" onclick="zoomToFullExtend(); return false;">'.L('map_zoomtoall').'</a> | '.L('map_jumpto').' <select id="zoomto" name="zoomto" size="1" onchange="gmapPan(this.value);">';
          $str .= '<option value="'.$qtg_gcenter.'"> </option>';
          $i=0;
          foreach($arrCoord as $aPoint)
          {
          $str .= '<option value="'.$aPoint['y'].','.$aPoint['x'].'">'.$aPoint['title'].'</option>';
          $i++; if ( $i>20 ) break;
          }
          $str .= '</select></p>';
        }
        $map = '<div class="gmap'.($qtg_mapontop ? ' top' : ' bottom').'">'.( empty($str) ? '' : $str ).PHP_EOL.'<div id="map_canvas"></div>'.PHP_EOL.'<p class="gmap bottom info">'.sprintf($L['map_items'], L('message',count($arrCoord)), L('message',$intMessages)).'</p></div>'.PHP_EOL;
      }
    }

    $out = $pagertop.($qtg_gkey && $qtg_mapontop ? $map.$out : $out.$map).$pagerbot; // Pagers messges and map

  }

  // Show messages and map
  echo $out;
  if ( $page===1 ) memcacheSet(QT.'-page1'.$_SESSION[QT]['language'],$out,false); // memcache page 1

}

// Other Part

if ( $qtg_checkip && strstr($qtg_checkip_list,($_SERVER['REMOTE_ADDR'])) )
{
  echo '<p style="text-align:center">',L('E_ip_denied'),'</p>';
  $qtg_actif=false;
}

if ( $qtg_actif )
{
  if ( $qtg_separateform || $qtg_gkey )
  {
    if ( $intMessages>2 ) echo '<div class="button sign"><a href="',Href('qtg_msg_add.php'),'">',L('Sign'.$qtg_type),'</a></div>';
  }
  else
  {
    $oPost = new cPost();
    $formaction = Href('qtg_msg_add.php');
    $certificate = makeFormCertificate('3bccd4c53e045fbe250d6c9d9e7ff7be'); // add certificate
    include 'qtg_form_msg.php';
  }
}
else
{
  echo '<p style="text-align:center">';
  if ( file_exists('config/wait-'.$_SESSION[QT]['language'].'.txt') ) { include 'config/wait-'.$_SESSION[QT]['language'].'.txt'; } else { echo 'Guestbook temporary closed. Please wait...'; }
  echo '</p>';
  if ( !empty($_SESSION['qtgAuth']) ) echo '<p style="text-align:center;font-size:8pt">[ Administrator: <a href="qtg_msg_add.php" style="font-size:8pt">add message</a> ]</p>';
}

// --------
// HTML END
// --------

$oHtml->scripts_end[] = '<script type="text/javascript">
var usercmd = document.getElementById("'.md5('cmd-'.$_SERVER['REMOTE_ADDR']).'");
if ( usercmd )
{
  var dataid = usercmd.getAttribute("data-id");
  var datatime = parseInt(usercmd.getAttribute("data-time"));
  if ( datatime>Math.round(Date.now()/1000) )
  {
    usercmd.style.display="inline";
    usercmd.innerHTML=" <a href=\"qtg_msg_edit.php?id=" + dataid + "\">['.L('Edit').'...]</a> <a href=\"qtg_msg_del.php?id=" + dataid + "\">['.L('Delete').'...]</a>";
    setTimeout( function(){usercmd.style.display="none"},300000 );
  }
}
</script>';

if ( $qtg_column_change ) $oHtml->scripts_end[] = '<script type="text/javascript">
function changeColumn(i) {
  var div = document.getElementById("messages");
  if ( div ) {
    div.setAttribute("class",i==1 ? "messages column" : "messages columns");
    setCookie("userview",i,100);
  }
}
// apply column user preference
var cvalue = getCookie("userview");
if ( cvalue=="" ) { cvalue='.($bCol ? '2' : '1').'; setCookie("userview",cvalue,100); }
changeColumn(cvalue);
</script>';

if ( $qtg_gkey && count($arrCoord)>0 )
{
  // check new map center
  $y = floatval(QTgety($qtg_gcenter));
  $x = floatval(QTgetx($qtg_gcenter));
  $aPoint = reset($arrCoord); // center on the first item
  if ( !empty($aPoint['y']) && !empty($aPoint['x']) )
  {
    $y=$aPoint['y'];
    $x=$aPoint['x'];
  }
  // update center
  $qtg_gcenter=$y.','.$x;
  // load markers
  $gmap_markers = array();
  $gmap_events = array();
  $gmap_functions = array();
  foreach($arrCoord as $aPoint)
  {
    if ( !empty($aPoint['y']) && !empty($aPoint['x']) )
    {
    $gmap_markers[] = QTgmapMarker($aPoint['y'].','.$aPoint['x'],false,$qtg_gsymbol,$aPoint['title'],$aPoint['info']);
    }
  }
  $gmap_functions[] = '
  function zoomToFullExtend()
  {
    if ( markers.length<2 ) return;
    var bounds = new google.maps.LatLngBounds();
    for (var i=markers.length-1; i>=0; i--) bounds.extend(markers[i].getPosition());
    map.fitBounds(bounds);
  }
  function showLocation(address)
  {
    if ( infowindow ) infowindow.close();
    geocoder.geocode( { "address": address}, function(results, status) {
      if (status == google.maps.GeocoderStatus.OK)
      {
        map.setCenter(results[0].geometry.location);
        if ( marker )
        {
          marker.setPosition(results[0].geometry.location);
        } else {
          marker = new google.maps.Marker({map: map, position: results[0].geometry.location, draggable: true, animation: google.maps.Animation.DROP, title: "Move to define the default map center"});
        }
      } else {
        alert("Geocode was not successful for the following reason: " + status);
      }
    });
  }

  ';
  include 'qtg_map_load.php';
}

// end content
echo '
</div>
';

include 'qtg_inc_ft.php';